@@ -8,9 +8,11 @@ from .current_flow_betweenness_subset import *
 from .degree_alg import *
 from .dispersion import *
 from .eigenvector import *
+from .group import *
 from .harmonic import *
 from .katz import *
 from .load import *
 from .reaching import *
 from .percolation import *
 from .second_order import *
+from .voterank_alg import *
