@@ -1,5 +1,5 @@
 #-*- coding: utf-8 -*-
-#    Copyright (C) 2011 by 
+#    Copyright (C) 2011 by
 #    Conrad Lee <conradlee@gmail.com>
 #    Aric Hagberg <hagberg@lanl.gov>
 #    All rights reserved.
@@ -10,6 +10,7 @@ __author__ = """\n""".join(['Conrad Lee <conradlee@gmail.com>',
                             'Aric Hagberg <aric.hagberg@gmail.com>'])
 __all__ = ['k_clique_communities']
 
+
 def k_clique_communities(G, k, cliques=None):
     """Find k-clique communities in graph using the percolation method.
 
@@ -23,7 +24,7 @@ def k_clique_communities(G, k, cliques=None):
     k : int
        Size of smallest clique
 
-    cliques: list or generator       
+    cliques: list or generator
        Precomputed cliques (use networkx.find_cliques(G))
 
     Returns
@@ -37,7 +38,7 @@ def k_clique_communities(G, k, cliques=None):
     >>> K5 = nx.convert_node_labels_to_integers(G,first_label=2)
     >>> G.add_edges_from(K5.edges())
     >>> c = list(k_clique_communities(G, 4))
-    >>> list(c[0])
+    >>> sorted(list(c[0]))
     [0, 1, 2, 3, 4, 5, 6]
     >>> list(k_clique_communities(G, 6))
     []
@@ -45,12 +46,12 @@ def k_clique_communities(G, k, cliques=None):
     References
     ----------
     .. [1] Gergely Palla, Imre Derényi, Illés Farkas1, and Tamás Vicsek,
-       Uncovering the overlapping community structure of complex networks 
+       Uncovering the overlapping community structure of complex networks
        in nature and society Nature 435, 814-818, 2005,
        doi:10.1038/nature03607
     """
     if k < 2:
-        raise nx.NetworkXError("k=%d, k must be greater than 1."%k)
+        raise nx.NetworkXError("k=%d, k must be greater than 1." % k)
     if cliques is None:
         cliques = nx.find_cliques(G)
     cliques = [frozenset(c) for c in cliques if len(c) >= k]
@@ -74,6 +75,7 @@ def k_clique_communities(G, k, cliques=None):
     for component in nx.connected_components(perc_graph):
         yield(frozenset.union(*component))
 
+
 def _get_adjacent_cliques(clique, membership_dict):
     adjacent_cliques = set()
     for n in clique:
