@@ -13,15 +13,14 @@
 module.
 
 """
-from nose.tools import assert_equal
-from nose.tools import raises
+import pytest
 
 import networkx as nx
 from networkx.algorithms.community import kernighan_lin_bisection
 
 
 def assert_partition_equal(x, y):
-    assert_equal(set(map(frozenset, x)), set(map(frozenset, y)))
+    assert set(map(frozenset, x)) == set(map(frozenset, y))
 
 
 def test_partition():
@@ -30,18 +29,24 @@ def test_partition():
     assert_partition_equal(C, [{0, 1, 2}, {3, 4, 5}])
 
 
-@raises(nx.NetworkXError)
+def test_seed_argument():
+    G = nx.barbell_graph(3, 0)
+    C = kernighan_lin_bisection(G, seed=1)
+    assert_partition_equal(C, [{0, 1, 2}, {3, 4, 5}])
+
+
 def test_non_disjoint_partition():
-    G = nx.barbell_graph(3, 0)
-    partition = ({0, 1, 2}, {2, 3, 4, 5})
-    kernighan_lin_bisection(G, partition)
+    with pytest.raises(nx.NetworkXError):
+        G = nx.barbell_graph(3, 0)
+        partition = ({0, 1, 2}, {2, 3, 4, 5})
+        kernighan_lin_bisection(G, partition)
 
 
-@raises(nx.NetworkXError)
 def test_too_many_blocks():
-    G = nx.barbell_graph(3, 0)
-    partition = ({0, 1}, {2}, {3, 4, 5})
-    kernighan_lin_bisection(G, partition)
+    with pytest.raises(nx.NetworkXError):
+        G = nx.barbell_graph(3, 0)
+        partition = ({0, 1}, {2}, {3, 4, 5})
+        kernighan_lin_bisection(G, partition)
 
 
 def test_multigraph():
