@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#    Copyright (C) 2006-2018 by
+#    Copyright (C) 2006-2019 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -19,8 +19,8 @@ In general, these functions do not check for acyclic-ness, so it is up
 to the user to check for that.
 """
 
-from collections import defaultdict
-from fractions import gcd
+from collections import defaultdict, deque
+from math import gcd
 from functools import partial
 from itertools import chain
 from itertools import product
@@ -28,20 +28,23 @@ from itertools import starmap
 import heapq
 
 import networkx as nx
+from networkx.algorithms.traversal.breadth_first_search import \
+    descendants_at_distance
 from networkx.generators.trees import NIL
 from networkx.utils import arbitrary_element
 from networkx.utils import consume
 from networkx.utils import pairwise
-from networkx.utils import generate_unique_node
 from networkx.utils import not_implemented_for
 
 __all__ = ['descendants',
            'ancestors',
            'topological_sort',
            'lexicographical_topological_sort',
+           'all_topological_sorts',
            'is_directed_acyclic_graph',
            'is_aperiodic',
            'transitive_closure',
+           'transitive_closure_dag',
            'transitive_reduction',
            'antichains',
            'dag_longest_path',
@@ -52,7 +55,7 @@ chaini = chain.from_iterable
 
 
 def descendants(G, source):
-    """Return all nodes reachable from `source` in `G`.
+    """Returns all nodes reachable from `source` in `G`.
 
     Parameters
     ----------
@@ -72,7 +75,7 @@ def descendants(G, source):
 
 
 def ancestors(G, source):
-    """Return all nodes having a path to `source` in `G`.
+    """Returns all nodes having a path to `source` in `G`.
 
     Parameters
     ----------
@@ -102,7 +105,7 @@ def has_cycle(G):
 
 
 def is_directed_acyclic_graph(G):
-    """Return True if the graph `G` is a directed acyclic graph (DAG) or
+    """Returns True if the graph `G` is a directed acyclic graph (DAG) or
     False if not.
 
     Parameters
@@ -118,7 +121,7 @@ def is_directed_acyclic_graph(G):
 
 
 def topological_sort(G):
-    """Return a generator of nodes in topologically sorted order.
+    """Returns a generator of nodes in topologically sorted order.
 
     A topological sort is a nonunique permutation of the nodes such that an
     edge from u to v implies that u appears before v in the topological sort
@@ -143,7 +146,7 @@ def topological_sort(G):
     NetworkXUnfeasible
         If `G` is not a directed acyclic graph (DAG) no topological sort exists
         and a :exc:`NetworkXUnfeasible` exception is raised.  This can also be
-        raised if `G` is changed while the returned iterator is being processed.
+        raised if `G` is changed while the returned iterator is being processed
 
     RuntimeError
         If `G` is changed while the returned iterator is being processed.
@@ -156,6 +159,16 @@ def topological_sort(G):
     >>> list(reversed(list(nx.topological_sort(DG))))
     [3, 2, 1]
 
+    If your DiGraph naturally has the edges representing tasks/inputs
+    and nodes representing people/processes that initiate tasks, then
+    topological_sort is not quite what you need. You will have to change
+    the tasks to nodes with dependence reflected by edges. The result is
+    a kind of topological sort of the edges. This can be done
+    with :func:`networkx.line_graph` as follows:
+
+    >>> list(nx.topological_sort(nx.line_graph(DG)))
+    [(1, 2), (2, 3)]
+
     Notes
     -----
     This algorithm is based on a description and proof in
@@ -199,7 +212,7 @@ def topological_sort(G):
 
 
 def lexicographical_topological_sort(G, key=None):
-    """Return a generator of nodes in lexicographically topologically sorted
+    """Returns a generator of nodes in lexicographically topologically sorted
     order.
 
     A topological sort is a nonunique permutation of the nodes such that an
@@ -229,7 +242,7 @@ def lexicographical_topological_sort(G, key=None):
     NetworkXUnfeasible
         If `G` is not a directed acyclic graph (DAG) no topological sort exists
         and a :exc:`NetworkXUnfeasible` exception is raised.  This can also be
-        raised if `G` is changed while the returned iterator is being processed.
+        raised if `G` is changed while the returned iterator is being processed
 
     RuntimeError
         If `G` is changed while the returned iterator is being processed.
@@ -249,14 +262,17 @@ def lexicographical_topological_sort(G, key=None):
        *Introduction to Algorithms - A Creative Approach.* Addison-Wesley.
     """
     if not G.is_directed():
-        raise nx.NetworkXError(
-            "Topological sort not defined on undirected graphs.")
+        msg = "Topological sort not defined on undirected graphs."
+        raise nx.NetworkXError(msg)
 
     if key is None:
-        def key(x): return x
+        def key(node):
+            return node
+
+    nodeid_map = {n: i for i, n in enumerate(G)}
 
     def create_tuple(node):
-        return key(node), node
+        return key(node), nodeid_map[node], node
 
     indegree_map = {v: d for v, d in G.in_degree() if d > 0}
     # These nodes have zero indegree and ready to be returned.
@@ -264,7 +280,7 @@ def lexicographical_topological_sort(G, key=None):
     heapq.heapify(zero_indegree)
 
     while zero_indegree:
-        _, node = heapq.heappop(zero_indegree)
+        _, _, node = heapq.heappop(zero_indegree)
 
         if node not in G:
             raise RuntimeError("Graph changed during iteration")
@@ -280,12 +296,132 @@ def lexicographical_topological_sort(G, key=None):
         yield node
 
     if indegree_map:
-        raise nx.NetworkXUnfeasible("Graph contains a cycle or graph changed "
-                                    "during iteration")
+        msg = "Graph contains a cycle or graph changed during iteration"
+        raise nx.NetworkXUnfeasible(msg)
+
+
+@not_implemented_for('undirected')
+def all_topological_sorts(G):
+    """Returns a generator of _all_ topological sorts of the directed graph G.
+
+    A topological sort is a nonunique permutation of the nodes such that an
+    edge from u to v implies that u appears before v in the topological sort
+    order.
+
+    Parameters
+    ----------
+    G : NetworkX DiGraph
+        A directed graph
+
+    Returns
+    -------
+    generator
+        All topological sorts of the digraph G
+
+    Raises
+    ------
+    NetworkXNotImplemented
+        If `G` is not directed
+    NetworkXUnfeasible
+        If `G` is not acyclic
+
+    Examples
+    --------
+    To enumerate all topological sorts of directed graph:
+
+    >>> DG = nx.DiGraph([(1, 2), (2, 3), (2, 4)])
+    >>> list(nx.all_topological_sorts(DG))
+    [[1, 2, 4, 3], [1, 2, 3, 4]]
+
+    Notes
+    -----
+    Implements an iterative version of the algorithm given in [1].
+
+    References
+    ----------
+    .. [1] Knuth, Donald E., Szwarcfiter, Jayme L. (1974).
+       "A Structured Program to Generate All Topological Sorting Arrangements"
+       Information Processing Letters, Volume 2, Issue 6, 1974, Pages 153-157,
+       ISSN 0020-0190,
+       https://doi.org/10.1016/0020-0190(74)90001-5.
+       Elsevier (North-Holland), Amsterdam
+    """
+    if not G.is_directed():
+        raise nx.NetworkXError(
+            "Topological sort not defined on undirected graphs.")
+
+    # the names of count and D are chosen to match the global variables in [1]
+    # number of edges originating in a vertex v
+    count = dict(G.in_degree())
+    # vertices with indegree 0
+    D = deque([v for v, d in G.in_degree() if d == 0])
+    # stack of first value chosen at a position k in the topological sort
+    bases = []
+    current_sort = []
+
+    # do-while construct
+    while True:
+        assert all([count[v] == 0 for v in D])
+
+        if len(current_sort) == len(G):
+            yield list(current_sort)
+
+            # clean-up stack
+            while len(current_sort) > 0:
+                assert len(bases) == len(current_sort)
+                q = current_sort.pop()
+
+                # "restores" all edges (q, x)
+                # NOTE: it is important to iterate over edges instead
+                # of successors, so count is updated correctly in multigraphs
+                for _, j in G.out_edges(q):
+                    count[j] += 1
+                    assert count[j] >= 0
+                # remove entries from D
+                while len(D) > 0 and count[D[-1]] > 0:
+                    D.pop()
+
+                # corresponds to a circular shift of the values in D
+                # if the first value chosen (the base) is in the first
+                # position of D again, we are done and need to consider the
+                # previous condition
+                D.appendleft(q)
+                if D[-1] == bases[-1]:
+                    # all possible values have been chosen at current position
+                    # remove corresponding marker
+                    bases.pop()
+                else:
+                    # there are still elements that have not been fixed
+                    # at the current position in the topological sort
+                    # stop removing elements, escape inner loop
+                    break
+
+        else:
+            if len(D) == 0:
+                raise nx.NetworkXUnfeasible("Graph contains a cycle.")
+
+            # choose next node
+            q = D.pop()
+            # "erase" all edges (q, x)
+            # NOTE: it is important to iterate over edges instead
+            # of successors, so count is updated correctly in multigraphs
+            for _, j in G.out_edges(q):
+                count[j] -= 1
+                assert count[j] >= 0
+                if count[j] == 0:
+                    D.append(j)
+            current_sort.append(q)
+
+            # base for current position might _not_ be fixed yet
+            if len(bases) < len(current_sort):
+                bases.append(q)
+
+        if len(bases) == 0:
+            break
 
 
 def is_aperiodic(G):
-    """Return True if `G` is aperiodic.
+    """Returns True if `G` is aperiodic.
 
     A directed graph is aperiodic if there is no integer k > 1 that
     divides the length of every cycle in the graph.
@@ -326,7 +462,7 @@ def is_aperiodic(G):
     levels = {s: 0}
     this_level = [s]
     g = 0
-    l = 1
+    lev = 1
     while this_level:
         next_level = []
         for u in this_level:
@@ -335,9 +471,9 @@ def is_aperiodic(G):
                     g = gcd(g, levels[u] - levels[v] + 1)
                 else:  # Tree Edge
                     next_level.append(v)
-                    levels[v] = l
+                    levels[v] = lev
         this_level = next_level
-        l += 1
+        lev += 1
     if len(levels) == len(G):  # All nodes in tree
         return g == 1
     else:
@@ -345,18 +481,30 @@ def is_aperiodic(G):
 
 
 @not_implemented_for('undirected')
-def transitive_closure(G):
+def transitive_closure(G, reflexive=False):
     """ Returns transitive closure of a directed graph
 
     The transitive closure of G = (V,E) is a graph G+ = (V,E+) such that
-    for all v,w in V there is an edge (v,w) in E+ if and only if there
-    is a non-null path from v to w in G.
+    for all v, w in V there is an edge (v, w) in E+ if and only if there
+    is a path from v to w in G.
+
+    Handling of paths from v to v has some flexibility within this definition.
+    A reflexive transitive closure creates a self-loop for the path
+    from v to v of length 0. The usual transitive closure creates a
+    self-loop only if a cycle exists (a path from v to v with length > 0).
+    We also allow an option for no self-loops.
 
     Parameters
     ----------
     G : NetworkX DiGraph
         A directed graph
-
+    reflexive : Bool or None, optional (default: False)
+        Determines when cycles create self-loops in the Transitive Closure.
+        If True, trivial cycles (length 0) create self-loops. The result
+        is a reflexive tranistive closure of G.
+        If False (the default) non-trivial cycles create self-loops.
+        If None, self-loops are not created.
+        
     Returns
     -------
     NetworkX DiGraph
@@ -371,11 +519,75 @@ def transitive_closure(G):
     ----------
     .. [1] http://www.ics.uci.edu/~eppstein/PADS/PartialOrder.py
 
+    TODO this function applies to all directed graphs and is probably misplaced
+         here in dag.py
     """
+    if reflexive is None:
+        TC = G.copy()
+        for v in G:
+            edges = ((v, u) for u in nx.dfs_preorder_nodes(G, v) if v != u)
+            TC.add_edges_from(edges)
+        return TC
+    if reflexive is True:
+        TC = G.copy()
+        for v in G:
+            edges = ((v, u) for u in nx.dfs_preorder_nodes(G, v))
+            TC.add_edges_from(edges)
+        return TC
+    # reflexive is False
     TC = G.copy()
     for v in G:
-        TC.add_edges_from((v, u) for u in nx.dfs_preorder_nodes(G, source=v)
-                          if v != u)
+        edges = ((v, w) for u, w in nx.edge_dfs(G, v))
+        TC.add_edges_from(edges)
+    return TC
+
+
+@not_implemented_for('undirected')
+def transitive_closure_dag(G, topo_order=None):
+    """ Returns the transitive closure of a directed acyclic graph.
+
+    This function is faster than the function `transitive_closure`, but fails
+    if the graph has a cycle.
+
+    The transitive closure of G = (V,E) is a graph G+ = (V,E+) such that
+    for all v, w in V there is an edge (v, w) in E+ if and only if there
+    is a non-null path from v to w in G.
+
+    Parameters
+    ----------
+    G : NetworkX DiGraph
+        A directed acyclic graph (DAG)
+
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
+    Returns
+    -------
+    NetworkX DiGraph
+        The transitive closure of `G`
+
+    Raises
+    ------
+    NetworkXNotImplemented
+        If `G` is not directed
+    NetworkXUnfeasible
+        If `G` has a cycle
+
+    Notes
+    -----
+    This algorithm is probably simple enough to be well-known but I didn't find
+    a mention in the literature.
+    """
+    if topo_order is None:
+        topo_order = list(topological_sort(G))
+
+    TC = G.copy()
+
+    # idea: traverse vertices following a reverse topological order, connecting
+    # each vertex to its descendants at distance 2 as we go
+    for v in reversed(topo_order):
+        TC.add_edges_from((v, u) for u in descendants_at_distance(TC, v, 2))
+
     return TC
 
 
@@ -409,20 +621,29 @@ def transitive_reduction(G):
 
     """
     if not is_directed_acyclic_graph(G):
-        raise nx.NetworkXError(
-            "Transitive reduction only uniquely defined on directed acyclic graphs.")
+        msg = "Directed Acyclic Graph required for transitive_reduction"
+        raise nx.NetworkXError(msg)
     TR = nx.DiGraph()
     TR.add_nodes_from(G.nodes())
+    descendants = {}
+    # count before removing set stored in descendants
+    check_count = dict(G.in_degree)
     for u in G:
-        u_edges = set(G[u])
+        u_nbrs = set(G[u])
         for v in G[u]:
-            u_edges -= {y for x, y in nx.dfs_edges(G, v)}
-        TR.add_edges_from((u, v) for v in u_edges)
+            if v in u_nbrs:
+                if v not in descendants:
+                    descendants[v] = {y for x, y in nx.dfs_edges(G, v)}
+                u_nbrs -= descendants[v]
+            check_count[v] -= 1
+            if check_count[v] == 0:
+                del descendants[v]
+        TR.add_edges_from((u, v) for v in u_nbrs)
     return TR
 
 
 @not_implemented_for('undirected')
-def antichains(G):
+def antichains(G, topo_order=None):
     """Generates antichains from a directed acyclic graph (DAG).
 
     An antichain is a subset of a partially ordered set such that any
@@ -433,6 +654,9 @@ def antichains(G):
     G : NetworkX DiGraph
         A directed acyclic graph (DAG)
 
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
     Returns
     -------
     generator object
@@ -458,8 +682,12 @@ def antichains(G):
     .. [1] Free Lattices, by R. Freese, J. Jezek and J. B. Nation,
        AMS, Vol 42, 1995, p. 226.
     """
-    TC = nx.transitive_closure(G)
-    antichains_stacks = [([], list(reversed(list(nx.topological_sort(G)))))]
+    if topo_order is None:
+        topo_order = list(nx.topological_sort(G))
+
+    TC = nx.transitive_closure_dag(G, topo_order)
+    antichains_stacks = [([], list(reversed(topo_order)))]
+
     while antichains_stacks:
         (antichain, stack) = antichains_stacks.pop()
         # Invariant:
@@ -475,7 +703,7 @@ def antichains(G):
 
 
 @not_implemented_for('undirected')
-def dag_longest_path(G, weight='weight', default_weight=1):
+def dag_longest_path(G, weight='weight', default_weight=1, topo_order=None):
     """Returns the longest path in a directed acyclic graph (DAG).
 
     If `G` has edges with `weight` attribute the edge data are used as
@@ -492,6 +720,9 @@ def dag_longest_path(G, weight='weight', default_weight=1):
     default_weight : int, optional
         The weight of edges that do not have a weight attribute
 
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
     Returns
     -------
     list
@@ -509,14 +740,20 @@ def dag_longest_path(G, weight='weight', default_weight=1):
     """
     if not G:
         return []
+
+    if topo_order is None:
+        topo_order = nx.topological_sort(G)
+
     dist = {}  # stores {v : (length, u)}
-    for v in nx.topological_sort(G):
+    for v in topo_order:
         us = [(dist[u][0] + data.get(weight, default_weight), u)
               for u, data in G.pred[v].items()]
+
         # Use the best predecessor if there is one and its distance is
         # non-negative, otherwise terminate.
         maxu = max(us, key=lambda x: x[0]) if us else (0, v)
         dist[v] = maxu if maxu[0] >= 0 else (0, v)
+
     u = None
     v = max(dist, key=lambda x: dist[x][0])
     path = []
@@ -524,6 +761,7 @@ def dag_longest_path(G, weight='weight', default_weight=1):
         path.append(v)
         u = v
         v = dist[v][1]
+
     path.reverse()
     return path
 
@@ -655,7 +893,7 @@ def dag_to_branching(G):
 
         >>> for source, nodes in sources.items():
         ...     for v in nodes:
-        ...         B.node[v].update(G.node[source])
+        ...         B.nodes[v].update(G.nodes[source])
 
     Notes
     -----
