@@ -9,7 +9,6 @@
 """Unit tests for the beam search functions."""
 from unittest import TestCase
 
-from nose.tools import assert_equal
 
 import networkx as nx
 
@@ -30,9 +29,9 @@ class TestBeamSearch(TestCase):
         # search terminates.
         G = nx.cycle_graph(4)
         edges = nx.bfs_beam_edges(G, 0, identity, width=1)
-        assert_equal(list(edges), [(0, 3), (3, 2)])
+        assert list(edges) == [(0, 3), (3, 2)]
 
     def test_wide(self):
         G = nx.cycle_graph(4)
         edges = nx.bfs_beam_edges(G, 0, identity, width=2)
-        assert_equal(list(edges), [(0, 3), (0, 1), (3, 2)])
+        assert list(edges) == [(0, 3), (0, 1), (3, 2)]
