@@ -161,7 +161,7 @@ def kruskal_mst_edges(G, minimum, weight='weight',
                     if ignore_nan:
                         continue
                     msg = "NaN found as an edge weight. Edge %s"
-                    raise ValueError(msg % ((u, v, f, k, d),))
+                    raise ValueError(msg % ((u, v, k, d),))
                 yield wt, u, v, k, d
     else:
         edges = G.edges(data=True)
@@ -235,15 +235,15 @@ def prim_mst_edges(G, minimum, weight='weight',
     push = heappush
     pop = heappop
 
-    nodes = list(G)
+    nodes = set(G)
     c = count()
 
     sign = 1 if minimum else -1
 
     while nodes:
-        u = nodes.pop(0)
+        u = nodes.pop()
         frontier = []
-        visited = [u]
+        visited = {u}
         if is_multigraph:
             for v, keydict in G.adj[u].items():
                 for k, d in keydict.items():
@@ -268,7 +268,7 @@ def prim_mst_edges(G, minimum, weight='weight',
                 W, _, u, v, k, d = pop(frontier)
             else:
                 W, _, u, v, d = pop(frontier)
-            if v in visited:
+            if v in visited or v not in nodes:
                 continue
             # Multigraphs need to handle edge keys in addition to edge data.
             if is_multigraph and keys:
@@ -282,8 +282,8 @@ def prim_mst_edges(G, minimum, weight='weight',
                 else:
                     yield u, v
             # update frontier
-            visited.append(v)
-            nodes.remove(v)
+            visited.add(v)
+            nodes.discard(v)
             if is_multigraph:
                 for w, keydict in G.adj[v].items():
                     if w in visited:
@@ -365,8 +365,8 @@ def minimum_spanning_edges(G, algorithm='kruskal', weight='weight',
     >>> G.add_edge(0, 3, weight=2)
     >>> mst = tree.minimum_spanning_edges(G, algorithm='kruskal', data=False)
     >>> edgelist = list(mst)
-    >>> sorted(edgelist)
-    [(0, 1), (1, 2), (2, 3)]
+    >>> sorted(sorted(e) for e in edgelist)
+    [[0, 1], [1, 2], [2, 3]]
 
     Find minimum spanning edges by Prim's algorithm
 
@@ -374,8 +374,8 @@ def minimum_spanning_edges(G, algorithm='kruskal', weight='weight',
     >>> G.add_edge(0, 3, weight=2)
     >>> mst = tree.minimum_spanning_edges(G, algorithm='prim', data=False)
     >>> edgelist = list(mst)
-    >>> sorted(edgelist)
-    [(0, 1), (1, 2), (2, 3)]
+    >>> sorted(sorted(e) for e in edgelist)
+    [[0, 1], [1, 2], [2, 3]]
 
     Notes
     -----
@@ -457,8 +457,8 @@ def maximum_spanning_edges(G, algorithm='kruskal', weight='weight',
     >>> G.add_edge(0, 3, weight=2)
     >>> mst = tree.maximum_spanning_edges(G, algorithm='kruskal', data=False)
     >>> edgelist = list(mst)
-    >>> sorted(edgelist)
-    [(0, 1), (0, 3), (1, 2)]
+    >>> sorted(sorted(e) for e in edgelist)
+    [[0, 1], [0, 3], [1, 2]]
 
     Find maximum spanning edges by Prim's algorithm
 
@@ -466,8 +466,8 @@ def maximum_spanning_edges(G, algorithm='kruskal', weight='weight',
     >>> G.add_edge(0, 3, weight=2) # assign weight 2 to edge 0-3
     >>> mst = tree.maximum_spanning_edges(G, algorithm='prim', data=False)
     >>> edgelist = list(mst)
-    >>> sorted(edgelist)
-    [(0, 1), (0, 3), (3, 2)]
+    >>> sorted(sorted(e) for e in edgelist)
+    [[0, 1], [0, 3], [2, 3]]
 
     Notes
     -----
@@ -542,7 +542,7 @@ def minimum_spanning_tree(G, weight='weight', algorithm='kruskal',
     """
     edges = minimum_spanning_edges(G, algorithm, weight, keys=True,
                                    data=True, ignore_nan=ignore_nan)
-    T = G.fresh_copy()  # Same graph class as G
+    T = G.__class__()  # Same graph class as G
     T.graph.update(G.graph)
     T.add_nodes_from(G.nodes.items())
     T.add_edges_from(edges)
@@ -563,7 +563,7 @@ def maximum_spanning_tree(G, weight='weight', algorithm='kruskal',
        Data key to use for edge weights.
 
     algorithm : string
-       The algorithm to use when finding a minimum spanning tree. Valid
+       The algorithm to use when finding a maximum spanning tree. Valid
        choices are 'kruskal', 'prim', or 'boruvka'. The default is
        'kruskal'.
 
@@ -575,7 +575,7 @@ def maximum_spanning_tree(G, weight='weight', algorithm='kruskal',
     Returns
     -------
     G : NetworkX Graph
-       A minimum spanning tree or forest.
+       A maximum spanning tree or forest.
 
 
     Examples
@@ -604,7 +604,7 @@ def maximum_spanning_tree(G, weight='weight', algorithm='kruskal',
     edges = maximum_spanning_edges(G, algorithm, weight, keys=True,
                                    data=True, ignore_nan=ignore_nan)
     edges = list(edges)
-    T = G.fresh_copy()  # Same graph class as G
+    T = G.__class__()  # Same graph class as G
     T.graph.update(G.graph)
     T.add_nodes_from(G.nodes.items())
     T.add_edges_from(edges)
