@@ -1,6 +1,6 @@
 """Laplacian matrix of graphs.
 """
-#    Copyright (C) 2004-2018 by
+#    Copyright (C) 2004-2019 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -14,12 +14,13 @@ __author__ = "\n".join(['Aric Hagberg <aric.hagberg@gmail.com>',
                         'Alejandro Weinstein <alejandro.weinstein@gmail.com>'])
 __all__ = ['laplacian_matrix',
            'normalized_laplacian_matrix',
-           'directed_laplacian_matrix']
+           'directed_laplacian_matrix',
+           'directed_combinatorial_laplacian_matrix']
 
 
 @not_implemented_for('directed')
 def laplacian_matrix(G, nodelist=None, weight='weight'):
-    """Return the Laplacian matrix of G.
+    """Returns the Laplacian matrix of G.
 
     The graph Laplacian is the matrix L = D - A, where
     A is the adjacency matrix and D is the diagonal matrix of node degrees.
@@ -50,6 +51,7 @@ def laplacian_matrix(G, nodelist=None, weight='weight'):
     --------
     to_numpy_matrix
     normalized_laplacian_matrix
+    laplacian_spectrum
     """
     import scipy.sparse
     if nodelist is None:
@@ -64,7 +66,7 @@ def laplacian_matrix(G, nodelist=None, weight='weight'):
 
 @not_implemented_for('directed')
 def normalized_laplacian_matrix(G, nodelist=None, weight='weight'):
-    r"""Return the normalized Laplacian matrix of G.
+    r"""Returns the normalized Laplacian matrix of G.
 
     The normalized graph Laplacian is the matrix
 
@@ -104,6 +106,7 @@ def normalized_laplacian_matrix(G, nodelist=None, weight='weight'):
     See Also
     --------
     laplacian_matrix
+    normalized_laplacian_spectrum
 
     References
     ----------
@@ -138,7 +141,7 @@ def normalized_laplacian_matrix(G, nodelist=None, weight='weight'):
 @not_implemented_for('multigraph')
 def directed_laplacian_matrix(G, nodelist=None, weight='weight',
                               walk_type=None, alpha=0.95):
-    r"""Return the directed Laplacian matrix of G.
+    r"""Returns the directed Laplacian matrix of G.
 
     The graph directed Laplacian is the matrix
 
@@ -179,14 +182,6 @@ def directed_laplacian_matrix(G, nodelist=None, weight='weight',
     L : NumPy array
       Normalized Laplacian of G.
 
-    Raises
-    ------
-    NetworkXError
-        If NumPy cannot be imported
-
-    NetworkXNotImplemnted
-        If G is not a DiGraph
-
     Notes
     -----
     Only implemented for DiGraphs
@@ -202,7 +197,140 @@ def directed_laplacian_matrix(G, nodelist=None, weight='weight',
        Annals of Combinatorics, 9(1), 2005
     """
     import scipy as sp
-    from scipy.sparse import identity, spdiags, linalg
+    from scipy.sparse import spdiags, linalg
+
+    P = _transition_matrix(G, nodelist=nodelist, weight=weight,
+                           walk_type=walk_type, alpha=alpha)
+
+    n, m = P.shape
+
+    evals, evecs = linalg.eigs(P.T, k=1)
+    v = evecs.flatten().real
+    p = v / v.sum()
+    sqrtp = sp.sqrt(p)
+    Q = spdiags(sqrtp, [0], n, n) * P * spdiags(1.0 / sqrtp, [0], n, n)
+    I = sp.identity(len(G))
+
+    return I - (Q + Q.T) / 2.0
+
+
+@not_implemented_for('undirected')
+@not_implemented_for('multigraph')
+def directed_combinatorial_laplacian_matrix(G, nodelist=None, weight='weight',
+                                            walk_type=None, alpha=0.95):
+    r"""Return the directed combinatorial Laplacian matrix of G.
+
+    The graph directed combinatorial Laplacian is the matrix
+
+    .. math::
+
+        L = \Phi - (\Phi P + P^T \Phi) / 2
+
+    where `P` is the transition matrix of the graph and and `\Phi` a matrix
+    with the Perron vector of `P` in the diagonal and zeros elsewhere.
+
+    Depending on the value of walk_type, `P` can be the transition matrix
+    induced by a random walk, a lazy random walk, or a random walk with
+    teleportation (PageRank).
+
+    Parameters
+    ----------
+    G : DiGraph
+       A NetworkX graph
+
+    nodelist : list, optional
+       The rows and columns are ordered according to the nodes in nodelist.
+       If nodelist is None, then the ordering is produced by G.nodes().
+
+    weight : string or None, optional (default='weight')
+       The edge data key used to compute each value in the matrix.
+       If None, then each edge has weight 1.
+
+    walk_type : string or None, optional (default=None)
+       If None, `P` is selected depending on the properties of the
+       graph. Otherwise is one of 'random', 'lazy', or 'pagerank'
+
+    alpha : real
+       (1 - alpha) is the teleportation probability used with pagerank
+
+    Returns
+    -------
+    L : NumPy array
+      Combinatorial Laplacian of G.
+
+    Notes
+    -----
+    Only implemented for DiGraphs
+
+    See Also
+    --------
+    laplacian_matrix
+
+    References
+    ----------
+    .. [1] Fan Chung (2005).
+       Laplacians and the Cheeger inequality for directed graphs.
+       Annals of Combinatorics, 9(1), 2005
+    """
+    from scipy.sparse import spdiags, linalg
+
+    P = _transition_matrix(G, nodelist=nodelist, weight=weight,
+                           walk_type=walk_type, alpha=alpha)
+
+    n, m = P.shape
+
+    evals, evecs = linalg.eigs(P.T, k=1)
+    v = evecs.flatten().real
+    p = v / v.sum()
+    Phi = spdiags(p, [0], n, n)
+
+    Phi = Phi.todense()
+
+    return Phi - (Phi*P + P.T*Phi) / 2.0
+
+
+def _transition_matrix(G, nodelist=None, weight='weight',
+                       walk_type=None, alpha=0.95):
+    """Returns the transition matrix of G.
+
+    This is a row stochastic giving the transition probabilities while
+    performing a random walk on the graph. Depending on the value of walk_type,
+    P can be the transition matrix induced by a random walk, a lazy random walk,
+    or a random walk with teleportation (PageRank).
+
+    Parameters
+    ----------
+    G : DiGraph
+       A NetworkX graph
+
+    nodelist : list, optional
+       The rows and columns are ordered according to the nodes in nodelist.
+       If nodelist is None, then the ordering is produced by G.nodes().
+
+    weight : string or None, optional (default='weight')
+       The edge data key used to compute each value in the matrix.
+       If None, then each edge has weight 1.
+
+    walk_type : string or None, optional (default=None)
+       If None, `P` is selected depending on the properties of the
+       graph. Otherwise is one of 'random', 'lazy', or 'pagerank'
+
+    alpha : real
+       (1 - alpha) is the teleportation probability used with pagerank
+
+    Returns
+    -------
+    P : NumPy array
+      transition matrix of G.
+
+    Raises
+    ------
+    NetworkXError
+        If walk_type not specified or alpha not in valid range
+    """
+
+    import scipy as sp
+    from scipy.sparse import identity, spdiags
     if walk_type is None:
         if nx.is_strongly_connected(G):
             if nx.is_aperiodic(G):
@@ -238,21 +366,10 @@ def directed_laplacian_matrix(G, nodelist=None, weight='weight',
     else:
         raise nx.NetworkXError("walk_type must be random, lazy, or pagerank")
 
-    evals, evecs = linalg.eigs(P.T, k=1)
-    v = evecs.flatten().real
-    p = v / v.sum()
-    sqrtp = sp.sqrt(p)
-    Q = spdiags(sqrtp, [0], n, n) * P * spdiags(1.0 / sqrtp, [0], n, n)
-    I = sp.identity(len(G))
-
-    return I - (Q + Q.T) / 2.0
-
-# fixture for nose tests
+    return P
 
 
+# fixture for pytest
 def setup_module(module):
-    from nose import SkipTest
-    try:
-        import numpy
-    except:
-        raise SkipTest("NumPy not available")
+    import pytest
+    numpy = pytest.importorskip('numpy')
