@@ -2,35 +2,30 @@
     Unit tests for yaml.
 """
 
-import os,tempfile
-from nose import SkipTest
-from nose.tools import assert_equal
+import os
+import tempfile
+import pytest
 
 import networkx as nx
 from networkx.testing import assert_edges_equal, assert_nodes_equal
 
+
 class TestYaml(object):
     @classmethod
-    def setupClass(cls):
-        global yaml
-        try:
-            import yaml
-        except ImportError:
-            raise SkipTest('yaml not available.')
+    def setup_class(cls):
+        cls.build_graphs()
 
-    def setUp(self):
-        self.build_graphs()
+    @classmethod
+    def build_graphs(cls):
+        cls.G = nx.Graph(name="test")
+        e = [('a', 'b'), ('b', 'c'), ('c', 'd'), ('d', 'e'), ('e', 'f'), ('a', 'f')]
+        cls.G.add_edges_from(e)
+        cls.G.add_node('g')
 
-    def build_graphs(self):
-        self.G = nx.Graph(name="test")
-        e = [('a','b'),('b','c'),('c','d'),('d','e'),('e','f'),('a','f')]
-        self.G.add_edges_from(e)
-        self.G.add_node('g')
+        cls.DG = nx.DiGraph(cls.G)
 
-        self.DG = nx.DiGraph(self.G)
-
-        self.MG = nx.MultiGraph()
-        self.MG.add_weighted_edges_from([(1,2,5),(1,2,5),(1,2,1),(3,3,42)])
+        cls.MG = nx.MultiGraph()
+        cls.MG.add_weighted_edges_from([(1, 2, 5), (1, 2, 5), (1, 2, 1), (3, 3, 42)])
 
     def assert_equal(self, G, data=False):
         (fd, fname) = tempfile.mkstemp()
@@ -42,7 +37,7 @@ class TestYaml(object):
 
         os.close(fd)
         os.unlink(fname)
-   
+
     def testUndirected(self):
         self.assert_equal(self.G, data=False)
 
@@ -52,3 +47,8 @@ class TestYaml(object):
     def testMultiGraph(self):
         self.assert_equal(self.MG, data=True)
 
+
+# fixture for pytest tests
+def setup_module(module):
+    import pytest
+    yaml = pytest.importorskip("yaml")
