@@ -1,5 +1,5 @@
-#    Copyright 2016-2017 NetworkX developers.
-#    Copyright (C) 2004-2017 by
+#    Copyright 2016-2019 NetworkX developers.
+#    Copyright (C) 2004-2019 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -8,7 +8,6 @@
 """
 Union-find data structure.
 """
-from itertools import groupby
 
 from networkx.utils import groups
 
@@ -91,6 +90,10 @@ class UnionFind:
             [['x', 'y'], ['z']]
 
         """
+        # Ensure fully pruned paths
+        for x in self.parents.keys():
+            _ = self[x] # Evaluated for side-effect only
+
         # TODO In Python 3.3+, this should be `yield from ...`.
         for block in groups(self.parents).values():
             yield block
